<?php
/**
*    This file is part of EvalWF Module - Automatic evaluation of forms was made by webform module.
*    Copyright (C) 2020-2022  University of Szeged
*
*    This EvalWF Module is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    Foobar is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Foobar.  If not, see <https://www.gnu.org/licenses/>.
*    it under the terms of the GNU General Public License as published by
*
* @file \Drupal\evalwf\Controller\EvalWFEvaluationListBuilder.php - Creates EvalWF settings list
*
* @author Tóthpál István
*/

namespace Drupal\evalwf\Controller;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\wevform\WebformSubmissionListBuilder;

/**
*  Class of ListBuilder for EvalWF settings
*/
class EvalWFEvaluationListBuilder extends ConfigEntityListBuilder {

  public $filter = null;

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['id'] = $this->t('ID');
    $header['wfid'] = $this->t('Webform');
    $header['sid'] = $this->t('Submission');
    $header['total_points'] = $this->t('Total points');
    $header['sent'] = $this->t('Sent');
    $header['sentto'] = $this->t('Sent to');
    $header['timestamp'] = $this->t('Timestamp');
    $header['uid'] = $this->t('UserID');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    if ( $this->filter==null  || ( $this->filter!=null && $entity->wfid==$this->filter)) {
      $row['id'] = $entity->id();
      $row['wfid'] = $entity->wfid;
      $row['sid'] = $entity->sid;
      $row['total_points'] = $entity->total_points;
      $row['sent'] = $entity->sent;
      $row['sentt'] = $entity->sentto;
      $row['timestamp'] = $entity->timestamp;
      $row['uid'] = \Drupal\user\Entity\User::load($entity->uid)->getAccountName();
      return $row + parent::buildRow($entity);
    }
    else {
      return null;
    }
  }

  public function renderfilter($filter) {
    $this->filter = $filter;

    $options[null]=null;
    $query = \Drupal::entityQuery('webform');
    $wfids = $query->execute();
    foreach($wfids as $key => $value ) {
      $options[$key] = $value;
    }
    $form['filter_table'] = [
      '#type' => 'table',
        0 => [
        'filter' => array(
          '#type' => 'select',
          '#title' => t('Select webform'),
          '#options' => $options,
          '#value' => $filter,
        ),
        'apply' => [
          '#type' => 'button',
          '#value' => 'Apply filter',
        ],
      ],
    ];

    $form = array_merge( $form, $this->render() );

    return $form;
  }

  public function render() {
    $form[] = parent::render();
    return $form;
  }
}

?>
