<?php
/**
*    This file is part of EvalWF Module - Automatic evaluation of forms was made by webform module.
*    Copyright (C) 2020-2022  University of Szeged
*
*    This EvalWF Module is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    Foobar is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Foobar.  If not, see <https://www.gnu.org/licenses/>.
*    it under the terms of the GNU General Public License as published by
*
* @file \Drupal\evalwf\Form\EvalWFEvaluationForm.php - Creates admin side forms to manage evaluations
*
* @author Tóthpál István
*
* @tutorial: "Creating a configuration entity type in Drupal 8" tutorial
*      @url: https://www.drupal.org/node/1809494
*/

namespace Drupal\evalwf\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Symfony\Component\Yaml\Yaml;

use Drupal\evalwf\Controller\EvalWFController;

/**
 * Form handler for the EvalWFForm.
 */
class EvalWFEvaluationForm extends EntityForm {

  /**
   *    Creates the form for evaluations
   *    @param form       - the current Form object
   *    @param form_state - the current FormStateInterface object
   *    @returns form array
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    switch ($form_state->getFormObject()->operation) {
      case 'view':
        $form['evaluation'] = [
          '#type' => 'details',
          '#title' => t('Evaluation data'),
          '#open' => TRUE,
          '#tree' => TRUE,
        ];
        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => $this->entity->id .'<br>',
          '#prefix' => 'ID: ',
        ];
        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => $this->entity->wfid,
          '#prefix' => '<b>Webform: ',
          '#suffix' => '</b><br>',
        ];
        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => $this->entity->sid,
          '#prefix' => '<b>Submission: ',
          '#suffix' => '</b><br><br>',
        ];

        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => $this->entity->total_points.'<br><br>',
          '#prefix' => '<b>Total points:</b> ',
        ];
        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => Yaml::dump(unserialize($this->entity->elements_points)),
          '#prefix' => '<b>Elements points:</b> <pre>',
          '#suffix' => '</pre><br>',
        ];
        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => Yaml::dump(unserialize($this->entity->conditions_data)),
          '#prefix' => '<b>Conditions data:</b> <pre>',
          '#suffix' => '</pre><br>',
        ];

        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => ($this->entity->sent ? 'TRUE':'FALSE' ).'<br>',
          '#prefix' => 'Sent in email: ',
        ];
        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => $this->entity->sentto.'<br><br>',
          '#prefix' => 'Sent to: ',
        ];

        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => $this->entity->timestamp.'<br>',
          '#prefix' => 'Timestamp: ',
        ];
        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' => User::load($this->entity->uid)->getAccountName().'<br><br>',
          '#prefix' => 'User: ',
        ];
        $form['evaluation'][] = [
          '#type' => 'markup',
          '#markup' =>  Yaml::dump(unserialize($this->entity->checksum)),
          '#prefix' => 'Checksums: <pre>',
          '#suffix' => '</pre><br>',
        ];
      break;
      case 'result':
        $mycontroller = new EvalWFController;
        $wf = \Drupal::entityTypeManager()->getStorage('webform')->load($this->entity->wfid);
        $wfs = \Drupal::entityTypeManager()->getStorage('webform_submission')->load($this->entity->sid);
        $form = $mycontroller->content( $wf, $wfs, true );
        break;
      case 'list':
        $this->entity->filter = (
          isset($form_state->getUserInput()['filter_table'][0]['filter']) ? $form_state->getUserInput()['filter_table'][0]['filter'] : null
        );
        $form[] = \Drupal::entityTypeManager()->getListBuilder('evalwf_evaluation')->renderfilter($this->entity->filter);

        $form[] = $this->BuildSubmissionsTable($this->entity->filter);

        break;
      default:
        $form['debug'] = [
              '#type' => 'markup',
              '#markup' => serialize($form_state).'<br>',
        ];
      break;
    }

    return $form;
  }

  /**
   *    This function runs the evaluation with selected submission ( button name contains )
   *    @param form       - the current Form object
   *    @param form_state - the current FormStateInterface object
   */
  function Evaluate( $form, $form_state){
    $op = $form_state->getTriggeringElement();
    if ($op && $op['#value']=='Evaluate') {
      $target = explode('-',$op['#name']);

      $mycontroller = new EvalWFController;
      $wf = \Drupal::entityTypeManager()->getStorage('webform')->load($target[1]);
      $wfs = \Drupal::entityTypeManager()->getStorage('webform_submission')->load($target[2]);
      // TODO: run evaluation without emailing / maybe show results
      $resultform = $mycontroller->content( $wf, $wfs, true );

      if ($resultform) {
        $button = array(
          '#type' => 'link',
          '#title' => t('View'),
          '#url' => \Drupal\Core\Url::fromRoute('entity.evalwf_evaluation.view_form', [ 'evalwf_evaluation' => $target[2] ] ),
        );
        \Drupal::messenger()->addMessage( t('The selected submission has evaluated...') );
        \Drupal::messenger()->addMessage( $button );
      }
    }
  }

  /**
   *    Builds a table with not evaluated submissions with filtering
   *    @param filter - the selected webform filter
   */
  function BuildSubmissionsTable($filter) {
    $query = \Drupal::entityQuery('evalwf_evaluation');
    $evaluations = $query->execute();

    $form['submissions-title'] = [
          '#type' => 'markup',
          '#markup' => '<h2>'.t('Available submissions:').'</h2>',
          '#prefix' => '<br><br>',
    ];
    $form['submissions'] = [
      '#type' => 'table',
      '#header' => [
        t('ID'),
        t('Webform'),
        t('Last changed'),
        t('User'),
        t('Operations'),
      ],
    ];
    $emptysize = count($form['submissions']);

    if ($filter)  {
      $query = \Drupal::entityQuery('webform_submission')->condition('webform_id', $filter)->sort('sid','DESC');
    }
    else {
      $query = \Drupal::entityQuery('webform_submission')->sort('sid','DESC');
    }
    $submissions = $query->execute();

    foreach ( $submissions as $key => $value) {
      if (!in_array($key,$evaluations)) {
        $webform_submission = \Drupal\webform\entity\WebformSubmission::load($key);
        $form['submissions'][$key] = [
          'id' => [ '#type' => 'markup', '#markup' => $value, ],
          'webform' => [ '#type' => 'markup', '#markup' => $webform_submission->bundle(), ],
          'changed' => [ '#type' => 'markup', '#markup' => date('Y-m-d H:i:s',$webform_submission->getChangedTime()) ],
          'user' => [ '#type' => 'markup', '#markup' => ($webform_submission->getOwner() ? $webform_submission->getOwner()->getAccountName() : '' ), ],
          'operations' => [
            '#type' => 'submit',
            '#value' => 'Evaluate',
            '#name' => 'Eval-'.$webform_submission->bundle().'-'.$key,
            '#submit' => array(
              '::Evaluate',
            ),
          ],
        ];
      }
    }

    if ( count($form['submissions']) <= $emptysize ) {
      $form = null;
    }
    return $form;
  }

  /**
   *    This function sets the action buttons/link on the form
   *    I'd like to remove these actions on add.
   *    @param form       - the current Form object
   *    @param form_state - the current FormStateInterface object
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    switch ($form_state->getFormObject()->operation) {
      case 'view':
      case 'list':
      case 'result':
        $actions = array();
        break;
      default:
        $actions = parent::actions($form, $form_state);
        break;
    }
    return $actions;
  }


}

?>
